import { HttpErrorResponse, HttpEvent, HttpHandler, HttpInterceptor, HttpRequest } from '@angular/common/http';
import { MatDialog } from '@angular/material/dialog';
import { Router } from '@angular/router';
import { Observable } from 'rxjs';
import { BaseAuthData } from '../models/base-auth-data.model';
import { BaseRole } from '../models/base-role.model';
import { BaseToken } from '../models/base-token.model';
import { JwtAuthService } from '../services/jwt-auth.service';
import * as i0 from "@angular/core";
/**
 * Interceptor that does error handling for http requests.
 */
export declare class HttpErrorInterceptor<AuthDataType extends BaseAuthData<TokenType, RoleValue, Role>, TokenType extends BaseToken, RoleValue extends string, Role extends BaseRole<RoleValue>, AuthServiceType extends JwtAuthService<AuthDataType, RoleValue, Role, TokenType>> implements HttpInterceptor {
    protected readonly router: Router;
    protected readonly authService: AuthServiceType;
    protected readonly dialog: MatDialog;
    protected readonly platformId: Object;
    /**
     * The message to display when the user has no internet connection.
     */
    protected readonly NO_INTERNET_CONNECTION_ERROR_MESSAGE: string;
    /**
     * The message to display when an error with CORS occurs.
     */
    protected readonly CORS_ERROR_MESSAGE: string;
    /**
     * All error codes for which the user should be logged out.
     */
    protected readonly logoutStatuses: number[];
    /**
     * Any api urls for which the user shouldn't be logged out.
     * Eg. The login page.
     */
    protected get apiUrlsWithNoLogout(): string[];
    constructor(router: Router, authService: AuthServiceType, dialog: MatDialog, platformId: Object);
    /**
     * The main method used by angular to intercept any http-requests with errors.
     * Displays an error message to the user and logs him out if requested.
     * @param request - The http-request that was intercepted.
     * @param next - The next http-handler in angular's chain.
     * @returns An Observable that is used by angular in the intercept chain.
     */
    intercept(request: HttpRequest<unknown>, next: HttpHandler): Observable<HttpEvent<unknown>>;
    /**
     * Checks if the user should be logged out after triggering the provided error.
     * @param error - The http-error that came from the api.
     * @param request - Data about the request that caused the error.
     * @returns Whether or not the current user should be logged out.
     */
    protected userShouldBeLoggedOut(error: HttpErrorResponse, request: HttpRequest<unknown>): boolean;
    /**
     * Checks if an dialog for the given error should be displayed to the user.
     * @param error - The http-error that was thrown.
     * @param request - Data about the request that caused the error.
     * @returns Whether or not an dialog should be displayed for the error.
     */
    protected errorDialogShouldBeDisplayed(error: HttpErrorResponse, request: HttpRequest<unknown>): boolean;
    /**
     * Gets the message from the HttpError.
     * Prefers the most nested one.
     * @param error - The http-error that was thrown.
     * @returns The message of the http-error.
     */
    protected getErrorDataMessage(error: HttpErrorResponse): string;
    /**
     * Checks if the provided error has something to do with CORS.
     * @param error - The error to check.
     * @returns Whether or not the provided error has something to do with CORS.
     */
    protected isCORSError(error: HttpErrorResponse): boolean;
    static ɵfac: i0.ɵɵFactoryDeclaration<HttpErrorInterceptor<any, any, any, any, any>, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<HttpErrorInterceptor<any, any, any, any, any>>;
}
