import { HttpEvent, HttpHandler, HttpInterceptor, HttpRequest } from '@angular/common/http';
import { InjectionToken } from '@angular/core';
import { Observable } from 'rxjs';
import { BaseAuthData } from '../models/base-auth-data.model';
import { BaseRole } from '../models/base-role.model';
import { BaseToken } from '../models/base-token.model';
import { JwtAuthService } from '../services/jwt-auth.service';
import * as i0 from "@angular/core";
/**
 * Used to define domains to which an jwt token should be added. This should be used to not send tokens e.g. To third party apis.
 */
export declare const NGX_JWT_INTERCEPTOR_ALLOWED_DOMAINS: InjectionToken<string[]>;
/**
 * Interceptor that sets the authorization header to the current token on every request.
 * Does nothing when the user isn't logged in.
 */
export declare class JwtInterceptor<AuthDataType extends BaseAuthData<TokenType, RoleValue, Role>, TokenType extends BaseToken, RoleValue extends string, Role extends BaseRole<RoleValue>, AuthServiceType extends JwtAuthService<AuthDataType, RoleValue, Role, TokenType>> implements HttpInterceptor {
    protected readonly authService: AuthServiceType;
    protected readonly allowedDomains?: string[] | undefined;
    constructor(authService: AuthServiceType, allowedDomains?: string[] | undefined);
    /**
     * The main method used by angular to intercept any http-requests and append the jwt.
     * @param request - The http-request that was intercepted.
     * @param next - The next http-handler in angular's chain.
     * @returns An Observable that is used by angular in the intercept chain.
     */
    intercept(request: HttpRequest<unknown>, next: HttpHandler): Observable<HttpEvent<unknown>>;
    /**
     * Check if the intercepted request is one of the special cases where no token is required.
     *
     * By default this is the case for the refresh-token and logout url.
     * @param request - The http-request that was intercepted.
     * @returns Whether or not the intercepted request is one of the special cases where no token is required.
     */
    protected requestDoesNotRequireToken(request: HttpRequest<unknown>): boolean;
    /**
     * Refreshes the token synchronous and sends the request afterwards.
     * @param request - The http-request that was intercepted.
     * @param next - The next http-handler in angular's chain.
     * @returns A promise of an unknown HttpEvent. Inside the interceptor you need to call "return from(this.refreshAndHandle(...))".
     */
    protected refreshAndHandle(request: HttpRequest<unknown>, next: HttpHandler): Observable<HttpEvent<unknown>>;
    /**
     * Checks whether or not the token needs to be refreshed.
     * @param request - The request to check.
     * @returns Whether or not the token needs to be refreshed.
     */
    protected tokenNeedsToBeRefreshed(request: HttpRequest<unknown>): boolean;
    /**
     * Checks whether or not the refresh token is expired.
     * @returns Whether or not the refresh token is expired.
     * @throws When there is no logged in user.
     */
    protected refreshTokenExpired(): boolean;
    /**
     * Checks if the request is to an allowed domain.
     * @param request - The request to check.
     * @returns Whether the request is to an allowed domain or not. Defaults to true if no allowed host names were provided.
     */
    protected requestIsToDisallowedDomain(request: HttpRequest<unknown>): boolean;
    /**
     * Gets a normalized domain from an url.
     * Is used for comparing the request url with the allowed domains array.
     * @param url - The url to get the domain from.
     * @returns The domain of the url.
     */
    protected getDomainFromUrl(url: string): string;
    static ɵfac: i0.ɵɵFactoryDeclaration<JwtInterceptor<any, any, any, any, any>, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<JwtInterceptor<any, any, any, any, any>>;
}
