import { HttpClient } from '@angular/common/http';
import { InjectionToken, NgZone } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { MatSnackBar } from '@angular/material/snack-bar';
import { Router } from '@angular/router';
import { BehaviorSubject } from 'rxjs';
import { SetupTwoFactorDialogConfig } from '../components/setup-two-factor-dialog/setup-two-factor-dialog.config';
import { TwoFactorDialogConfig } from '../components/two-factor-dialog/two-factor.dialog.config';
import { BaseAuthData, BiometricCredentials } from '../models/base-auth-data.model';
import { BaseRole } from '../models/base-role.model';
import { BaseToken } from '../models/base-token.model';
import { LoginData } from '../models/login-data.model';
import { AuthenticationResponse } from '../utilities/webauthn.utilities';
/**
 * Injection Token for the auth service.
 */
export declare const NGX_AUTH_SERVICE: InjectionToken<JwtAuthService<any, any, any, any>>;
/**
 * The response with the qr code url to enable 2fa.
 */
export interface TwoFactorUrlResponse {
    /**
     * The totp qr code url.
     */
    url: string;
}
/**
 * The response that is sent when logging in requires a 2fa code.
 */
export interface RequireTwoFactorResponse {
    /**
     * Notice that the login process requires a 2fa code.
     */
    require2fa: boolean;
}
/**
 * The Response vor verifying a password reset token.
 */
export interface VerifyResetTokenResponse {
    /**
     * Whether or not the provided reset token is valid.s.
     */
    isValid: boolean;
}
/**
 * The response sent to a user logging in when he is required to change his password.
 */
export interface RequirePasswordChangeResponse {
    /**
     * Whether or not the user is required to change his password.
     */
    requirePasswordChange: boolean;
}
/**
 * The base class for an authentication service.
 */
export declare abstract class JwtAuthService<AuthDataType extends BaseAuthData<TokenType, RoleValue, Role>, RoleValue extends string, Role extends BaseRole<RoleValue>, TokenType extends BaseToken> {
    protected readonly http: HttpClient;
    protected readonly snackbar: MatSnackBar;
    protected readonly zone: NgZone;
    protected readonly router: Router;
    protected readonly dialog: MatDialog;
    protected readonly platformId: Object;
    /**
     * The subject of the currently stored authData.
     */
    readonly authDataSubject: BehaviorSubject<AuthDataType | undefined>;
    /**
     * Subject to check if the access token is currently being refreshed.
     */
    readonly isRefreshingSubject: BehaviorSubject<boolean>;
    /**
     * The key for the authData saved in local storage.
     */
    readonly AUTH_DATA_KEY: string;
    /**
     * The key for the biometricCredentials saved in local storage.
     */
    readonly BIOMETRIC_CREDENTIALS_KEY: string;
    /**
     * The duration of the access token in milliseconds.
     * @default 3600000 // 1 hour
     */
    readonly ACCESS_TOKEN_DURATION_IN_MS: number;
    /**
     * The duration of the refresh token in milliseconds.
     * @default 8640000000 // 100 days
     */
    readonly REFRESH_TOKEN_DURATION_IN_MS: number;
    /**
     * The route for requesting a password change.
     * @default '/request-reset-password'
     */
    readonly REQUEST_RESET_PASSWORD_ROUTE: string;
    /**
     * The message to display inside a snackbar when the mail for resetting a password was sent successfully.
     * @default 'A Mail for changing your password is on its way'
     */
    readonly REQUEST_RESET_PASSWORD_SNACK_BAR_MESSAGE: string;
    /**
     * The message to display inside a snackbar when password was reset successfully.
     * @default 'Password changed successfully!'
     */
    readonly CONFIRM_RESET_PASSWORD_SNACK_BAR_MESSAGE: string;
    /**
     * The name of the custom header that is used to transport two factor codes.
     * @default 'X-Authorization-2FA'
     */
    readonly TWO_FACTOR_HEADER: string;
    /**
     * The default url for login requests.
     */
    abstract readonly API_LOGIN_URL: string;
    /**
     * The default url for logout requests.
     */
    abstract readonly API_LOGOUT_URL: string;
    /**
     * When the user tries to access a route for which he doesn't have the permission and is logged out
     * he gets redirected to this route afterwards.
     * @default '/login'
     */
    readonly ROUTE_AFTER_LOGOUT: string;
    /**
     * The default url for refresh token requests.
     */
    abstract readonly API_REFRESH_TOKEN_URL: string;
    /**
     * The default url for request reset password requests.
     */
    abstract readonly API_REQUEST_RESET_PASSWORD_URL: string;
    /**
     * The default url for confirm reset password requests.
     */
    abstract readonly API_CONFIRM_RESET_PASSWORD_URL: string;
    /**
     * The default url for verify password reset token requests.
     */
    abstract readonly API_VERIFY_RESET_PASSWORD_TOKEN_URL: string;
    /**
     * The api url for turning on two factor authentication. Needs to return a qr code url.
     */
    abstract readonly API_TURN_ON_TWO_FACTOR_URL: string;
    /**
     * The api url to confirm turning on two factor authentication.
     */
    abstract readonly API_CONFIRM_TURN_ON_TWO_FACTOR_URL: string;
    /**
     * The api url for turning off two factor authentication.
     */
    abstract readonly API_TURN_OFF_TWO_FACTOR_URL: string;
    /**
     * The api url for registering a biometric credential.
     */
    abstract readonly API_REGISTER_BIOMETRIC_CREDENTIAL: string;
    /**
     * The api url for confirming the registration of a biometric credential.
     */
    abstract readonly API_CONFIRM_REGISTER_BIOMETRIC_CREDENTIAL: string;
    /**
     * The api url for getting all possible biometric credentials.
     */
    abstract readonly API_GENERATE_BIOMETRIC_AUTHENTICATION_OPTIONS: string;
    /**
     * The api url for canceling the registration of a biometric credential.
     */
    abstract readonly API_CANCEL_REGISTER_BIOMETRIC_CREDENTIAL: string;
    /**
     * The currently stored authData value.
     * Contains at least the token.
     */
    get authData(): AuthDataType | undefined;
    set authData(value: AuthDataType | undefined);
    /**
     * The biometricCredentials saved in local storage.
     * This is separated from the auth data because it's also needed when the user is logged out.
     */
    get biometricCredentials(): BiometricCredentials[];
    set biometricCredentials(value: BiometricCredentials[] | undefined);
    /**
     * Whether or not the access token is currently being refreshed.
     */
    get isRefreshing(): boolean;
    constructor(http: HttpClient, snackbar: MatSnackBar, zone: NgZone, router: Router, dialog: MatDialog, platformId: Object);
    /**
     * Gets called right before auth data is set.
     * Can be used to transform some of the data.
     *
     * DEFAULT: When the api sends roles as a list of strings instead of Role objects,
     * they are transformed to role objects with displayName and value being the string send by the api.
     * @param authData - The auth data that should be set.
     * @returns The transformed auth data or undefined.
     */
    protected transformAuthDataBeforeSetting(authData: AuthDataType | undefined): AuthDataType | undefined;
    /**
     * Login a user.
     * @param loginData - The data that is sent to the server to login the user.
     * @returns A promise of the received authData.
     */
    login(loginData: LoginData | AuthenticationResponse): Promise<AuthDataType>;
    private openChangePasswordDialog;
    private isRequirePasswordChangeType;
    /**
     * Opens a two factor dialog with the given configuration data and returns the code that has been input.
     * @param data - Configuration data for the dialog.
     * @returns The input two factor code or undefined if the dialog was closed with cancel.
     */
    openInput2FADialog(data?: Partial<TwoFactorDialogConfig>): Promise<string | undefined>;
    private isAuthDataType;
    /**
     * Logout the current user.
     */
    logout(): Promise<void>;
    /**
     * Refreshes the token.
     */
    refreshToken(): Promise<void>;
    /**
     * Requests a new password from the server.
     * Should sent a reset-link to the given email with a one time short lived (~5 minutes) token.
     * @param email - The email of the user that wants to reset his password.
     */
    requestResetPassword(email: string): Promise<void>;
    /**
     * Confirms the reset of the password.
     * @param newPassword - The new password.
     * @param resetToken - The token from the email. Needed to authorize the password reset.
     */
    confirmResetPassword(newPassword: string, resetToken: string): Promise<void>;
    /**
     * Checks if the given reset token is valid.
     * @param resetToken - The token from the email. Needed to authorize the password reset.
     * @returns Whether or not the given token is valid.
     */
    isResetTokenValid(resetToken: string): Promise<boolean>;
    /**
     * Checks whether or not the currently logged in user has one of the provided roles.
     * @param allowedRolesValues - All roles that are allowed to do a certain thing.
     * @returns Whether or not the user has one of the provided allowed roles.
     */
    hasRole(allowedRolesValues: RoleValue[]): boolean;
    /**
     * Generates a qr code url to setup 2fa in eg. Google Authenticator.
     * @returns The response with the qr code url.
     */
    turnOn2FA(): Promise<TwoFactorUrlResponse>;
    /**
     * Opens the dialog to turn on 2fa. The dialog displays a qr code and an input to confirm with a two factor code.
     * @param data - Configuration data for the dialog.
     */
    openTurnOn2FADialog(data?: Partial<SetupTwoFactorDialogConfig>): void;
    /**
     * Confirms turning on two factor authentication.
     * Sends the provided two factor code to the configured endpoint.
     * @param twoFactorCode - The two factor code that the user generated using eg. Google Authenticator.
     */
    confirmTurnOn2FA(twoFactorCode: string): Promise<void>;
    /**
     * Turns off two factor authentication for the current user.
     */
    turnOff2FA(): Promise<void>;
    /**
     * Registers a new biometric credential for the currently logged in user.
     */
    registerBiometricCredential(): Promise<void>;
    /**
     * Tries to login with biometric authentication.
     */
    loginWithBiometricAuthentication(): Promise<void>;
}
